<?php
require_once('Hash.php');
class Common{
	static function checkMethod(&$obj, $method){
		if(!method_exists($obj, $method)){
			echo "The method: $method does not exist for this object: ";
			print_r($obj);
			exit;
		}
	}
	
	static function assocObj($arr, $key){
		$rarr = array();
		foreach($arr as $obj){ $rarr[$obj->$key] = $obj; }
		return $rarr;
	}
	
	static function objToAssoc($obj, $fields = false){
		$rarr = array();
		if($fields){
			foreach($fields as $field)
				$rarr[$field] = $obj->$field;
		}else{
			foreach($obj as $prop => $value)
				$rarr[$prop] = $value;
		}
		return $rarr;
	}
	
	static function sort2dBySubLength($arr, $key, $order){
		foreach($arr as $sub) $sc[] = count($sub[$key]);
		$order = $order == 'desc' ? SORT_DESC : SORT_ASC;
		array_multisort($sc, SORT_NUMERIC, $order, $arr);
		return $arr;
	}
	
	static function fileNameFromPath($path){
		$p_arr = explode("/", $path);
		return $p_arr[count($p_arr) - 1];
	}

	static function getImagePrefix($type){
		$arr = array(1 => "gif", 2 => "jpg", 3 => "png");
		return $arr[$type];
	}
	
	static function hash($str){
		$hasher = new PasswordHash(8, FALSE);
		return $hasher->HashPassword($str);
	}
	
	static function checkHash($str, $old_str){
		$hasher = new PasswordHash(8, FALSE);
		return $hasher->CheckPassword($str, $old_str);
	}
	
	/**
	* Resizes an image absolutely, it does not keep aspect ratio.
	* 
	* @param string $fileName The name of the image file.
	* @param integer $new_width The new width the image will get.
	* @param integer $new_height The new height the image will get.
	* @return boolean Returns false if it fails, otherwise true.
	*/
  	static function resizeDiscardAspect($fileName, $new_width, $new_height){
		//we retrieve the info from the current image
  		list($orig_width, $orig_height, $type) = getimagesize($fileName);
  		//we create a new image template
  		$image_p = imagecreatetruecolor($new_width, $new_height);
  		//we create a variable that will hold the new image
  		$image = null;
  		//only the three first of all the possible formats are supported, the original image is loaded if it is one of them
		switch($type){
		  	case 1: //GIF
		  		$image = imagecreatefromgif($fileName);
		  		break;
		  	case 2: //JPEG
		  		$image = imagecreatefromjpeg($fileName);
		  		break;
		  	case 3: //PNG
		  		$image = imagecreatefrompng($fileName);
		  		break;
		  	default:
		  		return false;
		  		break;
		}
		//we copy the resized image from the original into the new one and save the result as a jpeg   
		imagecopyresampled($image_p, $image, 0, 0, 0, 0, $new_width, $new_height, $orig_width, $orig_height);
		imagejpeg($image_p, $fileName, 100);
		return true;
	}
	
	/**
	 * This function is used to retrieve image dimensions for arbitrary scaling
	 * using constraints in both the x- and y-dimension.
	 *
	 * @param string $filename The path to the image.
	 * @param int $conx The maximum width of the scaled image.
	 * @param int $cony The maximum height of the scaled image.
	 * @param array &$member The array to store the new width and height in.
	 */
	static function constrainImage($filename, $conx, $cony){
		list($orig_width, $orig_height, $type) = getimagesize($filename);
		
		$new_width	=	0;
		$new_height	=	0;
		
		// for instance 0.66 = 125 / 188
		$con_ratio 		= $conx / $cony;
		
		// for instance  1.33 =  300 / 225
		$orig_ratio		= $orig_width / $orig_height;
		
		//if the new picture is laying and the original is standing or laying
		//"less", the original height has to lead
		if($con_ratio > $orig_ratio){
			$new_height = $cony;
			$new_width 	= round(( $cony * $orig_width) / $orig_height);
		}else if($con_ratio < $orig_ratio){
			$new_height = round(( $conx * $orig_height) / $orig_width);
			$new_width 	= $conx;
		}
		
		return array($new_width, $new_height);
	}
	
	static function reScaleImage($filename, $conx, $cony){
		if(empty($conx) || empty($cony))
			return;
		list($new_width, $new_height) = self::constrainImage($filename, $conx, $cony);
		return self::resizeDiscardAspect($filename, $new_width, $new_height);
	}
	
	/**
	 * Returns an array with all filenames of the files in the directory
	 *
	 * @param dir &$d the directory to parse
	 * @return array the array with filenames
	 */
  	static function dirAsArr(&$d){
  	  	$arr = array();
		for($i = 0; false !== ($entry = $d->read()); $i++){
			if($i > 1) 	
				$arr[] = $entry;
		}

		return $arr;
	}
	
	/**
	 * Same functionality as dirAsArr() but accepts a string representing the
	 * directory name.
	 *
	 * @uses FileDirExt::dirAsArr()
	 * @param string $strDir the directory path
	 * @return array the array with filenames
	 */
	static function listDir($strDir){
		$d = dir($strDir);
		return self::dirAsArr($d);
	}
	
	static function loadCtrl($ctrl){
		$name = ucfirst($ctrl)."Ctrl";
		if(!class_exists($name)){
			$file_name = "controllers/$name.php";
			if(!is_file($file_name)){
				echo "The controller does not exist";
				exit;
			}
	 		include_once($file_name);
		}
		$obj = new $name();
		$obj->init(ucfirst($ctrl));
		return $obj;
	} 
	
	

	static function keyValue($arr, $kname, $vname){
		foreach($arr as $sub)
			$rarr[ $sub[$kname] ] = $sub[ $vname ];
		return $rarr;
	}
	
	static function array_trim($arr){
		$rarr = array();
		foreach($arr as $key => $value){
			if(!empty($value))
				$rarr[$key] = $value;
		}
		return $rarr;
	}
	
	static function index(){
		$req 		= array_slice(explode('/', $_SERVER['REQUEST_URI']), 1);
		$base_count = empty($GLOBALS['base_dir']) ? 0 : count(explode('/', $GLOBALS['base_dir']));
		$cur_ctrl 	= $req[$base_count];
		$cur_meth 	= $req[$base_count + 1];
		$args_start = $base_count + 2;
		if(empty($cur_ctrl) || empty($cur_meth)){ echo "No ctrl or method"; exit; }
		$args		= self::array_trim(array_slice($req, $args_start));
		if(count($args) % 2 != 0){ echo "key/missing value"; exit; }
		for($i = 0; $i < count($args) - 1; $i += 2)
			$_GET[ $args[$i] ] = $args[$i + 1];
			
		$smarty 	= new SmartyDoctrine();
		$obj 		= self::loadCtrl($cur_ctrl);
		$smarty->assign('site_dir', "/{$GLOBALS['base_dir']}/");
		$smarty->assign('content', $obj->$cur_meth());
		$smarty->display('index.tpl');
	}
}